/*
 * Copyright 2018 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.drools.workbench.screens.guided.dtable.backend.server;

import java.util.List;

import javax.annotation.PostConstruct;
import javax.inject.Inject;

import org.drools.workbench.screens.guided.dtable.model.GuidedDecisionTableEditorContent;
import org.drools.workbench.screens.guided.dtable.service.GuidedDecisionTableEditorService;
import org.drools.workbench.screens.guided.dtable.service.GuidedDecisionTableGraphSaveAndRenameService;
import org.guvnor.common.services.shared.metadata.model.Metadata;
import org.uberfire.backend.vfs.Path;
import org.uberfire.ext.editor.commons.backend.service.SaveAndRenameServiceImpl;

public class GuidedDecisionTableGraphSaveAndRenameServiceImpl implements GuidedDecisionTableGraphSaveAndRenameService {

    private GuidedDecisionTableEditorService editorService;

    private SaveAndRenameServiceImpl<List<GuidedDecisionTableEditorContent>, Metadata> saveAndRenameService;

    @Inject
    public GuidedDecisionTableGraphSaveAndRenameServiceImpl(final GuidedDecisionTableEditorService editorService,
                                                            final SaveAndRenameServiceImpl<List<GuidedDecisionTableEditorContent>, Metadata> saveAndRenameService) {
        this.editorService = editorService;
        this.saveAndRenameService = saveAndRenameService;
    }

    @PostConstruct
    public void init() {
        saveAndRenameService.init(this);
    }

    @Override
    public Path saveAndRename(final Path path,
                              final String newFileName,
                              final Metadata metadata,
                              final List<GuidedDecisionTableEditorContent> content,
                              final String comment) {
        return saveAndRenameService.saveAndRename(path, newFileName, metadata, content, comment);
    }

    @Override
    public Path rename(final Path path,
                       final String newName,
                       final String comment) {
        return editorService.rename(path, newName, comment);
    }

    @Override
    public Path save(final Path path,
                     final List<GuidedDecisionTableEditorContent> content,
                     final Metadata metadata,
                     final String comment) {

        content.forEach(c -> {
            editorService.save(c.getCurrentPath(), c.getModel(), c.getOverview().getMetadata(), comment);
        });

        return path;
    }
}
